#! /usr/bin/python

"""VLC Widget classes.

This module provides two helper classes, to ease the embedding of a
VLC component inside a pygtk application.

VLCWidget is a simple VLC widget.

DecoratedVLCWidget provides simple player controls.

"""

import gtk
import sys
import vlc
import settings
import os
import logging
import glob
from time import sleep
from time import strftime
from gettext import gettext as _
from PIL import Image


class VLCWidget(gtk.DrawingArea):
    """Simple VLC widget.

    Its player can be controlled through the 'player' attribute, which
    is a MediaControl instance.
    """
    def __init__(self, *p):
        gtk.DrawingArea.__init__(self)
        self.player=vlc.MediaControl(*p)
        def handle_embed(*p):
            if sys.platform == 'win32':
                xidattr='handle'
            else:
                xidattr='xid'
            self.player.set_visual(getattr(self.window, xidattr))
            return True
        self.connect("map-event", handle_embed)
        self.set_size_request(640, 480)


class DecoratedVLCWidget(gtk.VBox):
    """Decorated VLC widget.

    VLC widget decorated with a player control toolbar.

    Its player can be controlled through the 'player' attribute, which
    is a MediaControl instance.
    """
    def __init__(self, *p):
        self.first_run = True
        gtk.VBox.__init__(self)
        self._vlc_widget=VLCWidget(*p)
        self.player=self._vlc_widget.player
        self.pack_start(self._vlc_widget, expand=True)
        self.toolbar = self.get_player_control_toolbar()
        self.pack_start(self.toolbar, expand=False)

    def get_player_control_toolbar(self):
        """Return a player control toolbar
        """
        tb = gtk.HButtonBox()

        def on_snapshot(b):
            # Do foo to snapshot a picture
            if self.first_run:
                logging.debug("Starting video capture")
                self.player.start(0)
                self.first_run = False
                b.set_label("Click to take picture")
                return True
            else :
                logging.debug("Taking snapshot")
                b.set_sensitive(False)
                imageset = strftime("%Y-%m-%d_%H-%M-%S-")
                for i in xrange(1,5):
                    b.set_label("Say Cheese!")
                    while gtk.events_pending():
                        gtk.main_iteration(False)
                    sleep(1)
                    snap = self.player.snapshot(0)
                    imagefilename = imageset + str(i) + '.png'
                    imagepath = os.path.join(settings.store, imagefilename)
                    f = open(imagepath, 'w')
                    f.write(snap['data'])
                    f.close()
                    b.set_label("Snapshot " + str(i) + " Complete")
                    while gtk.events_pending():
                        gtk.main_iteration(False)
                    sleep(1)
                
                glob_pattern = settings.store + "/" + imageset+"*.png" 
                img_files = glob.glob(glob_pattern)

                # Get size of one of the images. Assumes all images are same
                # size

                im = Image.open(img_files[0])
                (imgx,imgy) = im.size
                mode = im.mode
                border = 5
                borderedge = 10
                finalx = 4 * imgx + border * 3 + 2 * borderedge
                finaly = imgy + 2 * borderedge

                im_final = Image.new(mode, (finalx, finaly), (255,255,255))
                for idx, img_file in enumerate(img_files):
                    im = Image.open(img_file)
                    im_final.paste(im, (borderedge + idx * (imgx + border) , borderedge))
                    os.remove(img_file)
                im_final.save(os.path.join(settings.store, imageset + ".jpg"))

                b.set_label("Click to take picture")
                b.set_sensitive(True)
                return True

        snapshot = gtk.Button(gtk.STOCK_MEDIA_PLAY)
        snapshot.set_label("Click to Start")
        snapshot.connect("clicked", on_snapshot)

        tb.pack_start(snapshot,expand=True, fill=True, padding=10)
        tb.show_all()
        return tb

class VideoPlayer:
    """Example video player.
    """
    def __init__(self):
        self.vlc = DecoratedVLCWidget()

    def popup(self):
        w=gtk.Window()
        w.add(self.vlc)
        w.show_all()
        w.connect("destroy", gtk.main_quit)
        return w
    
    def main(self, fname):
        self.vlc.player.set_mrl(fname)
        self.popup()

        # Control ends here and processing of event handling stuff begins
        #self.vlc.player._toolbar.get_children()
        gtk.main()


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    if sys.argv[1:]:
        vidsrc = sys.argv[1]
    else :
        print "Movie filename not provided. Using Capture device"
        vidsrc = "v4l2://"
       
    p=VideoPlayer()
    p.main(vidsrc)
